var platforms = [{
    site: 'ae',
    alias: '速卖通'
}, {
    site: 'wish',
    alias: 'wish'
}, {
    site: 'lazada',
    alias: 'lazada'
}, {
    site: 'ebay',
    alias: 'ebay'
}];
var chrome = browser;
// Create one test item for each context type.
var contexts = ["page", "selection", "link", "editable", "image"];
for (var i = 0; i < contexts.length; i++) {
    var context = contexts[i];

    chrome.contextMenus.create({
        "title": '分类采集',
        "contexts": [context],
        "onclick": function (info, tab) {
            fetch(info.pageUrl, tab, 'category');
        }
    });
    chrome.contextMenus.create({
        "title": '单品采集',
        "contexts": [context],
        "onclick": function (info, tab) {
            fetch(info.pageUrl, tab, 'single');
        }
    });
    chrome.contextMenus.create({
        "type": 'separator',
        "contexts": [context]
    });
    platforms.forEach(function (platform) {
        chrome.contextMenus.create({
            "title": '采集到' + platform.alias,
            "contexts": [context],
            "onclick": function (info, tab) {
                fetch(info.pageUrl, tab, 'single', platform.site); 
            }
        });
    });
}
var urlMap = {};
chrome.browserAction.onClicked.addListener(function (tab) {
    var pageUrl = tab.url;
    var type = urlMap[pageUrl];
    if (type) {
        fetch(pageUrl, tab, type);
    } else {
        $.getJSON('http://www.mangoerp.com/bridge/gather/type?url=' + pageUrl, function (res) {
            if (res.success) {
                fetch(pageUrl, tab, res.data.type);
            } else {
                alert (res.msg);
            }
        }).fail(function (xh4, textStatus) {
            alert (textStatus);
        });
    }
});

chrome.runtime.onMessage.addListener(function (request, sender) {
    if (request.ID !== 'background') {
        return;
    }
    /*
     * request.action 当前只有fetch
     * request.url  单个url
     * request.urls 多个url
     * request.type single|category
     * request.silent 当true时不提示
     *
     */
    var urls = [];
    switch (request.action) {
        case 'preload':
            //从content.js接收
            urlMap[sender.tab.url] = request.type;
            break;
        case 'fetch':
            //从content.js接收
            if (request.title) {
                sender.tab.title = request.title;
            }
            fetch(request.url || sender.tab.url, sender.tab, request.type, undefined, request.silent);
            break;
        case 'options':
            chrome.tabs.create({'url':chrome.extension.getURL('options.htm'), active:true});
            break;
        case 'mangoerp.fetch':
            //http://www.mangoerp.com 网页采集消息
            if (request.urls) {
                urls = request.urls;
            } 
            if (request.url) {
                urls.push(request.url);
            }
            urls.forEach(function (url, index) {
                if (request.type === 'single' && index >= 1) {
                    setTimeout(function () {
                        fetch(url, {title:url}, request.type);
                    }, 200);
                } else {
                    fetch(url, {title:url}, request.type);
                }
            });
            break;
        case 'mangoerp.printers':
            break;
        case 'mangoerp.print':
            break;
    }
});

//https://developer.chrome.com/extensions/webRequest#event-onCompleted
if (chrome.webRequest) {
    chrome.webRequest.onCompleted.addListener(function (detail) {
        //通知content.js处理handleLinks
        chrome.tabs.sendMessage(detail.tabId, {ID:'content', action:'handleLinks'});
    }, {
        urls: ['<all_urls>'],
        types: ['xmlhttprequest', 'script']
    });
}

function fetch(pageUrl, tab, type, site, silent) {
    if (type === 'category') {
        createMainTab();
        return;
    }
    if (chrome._mangoTabId) {
        //如果创建过maintab
        chrome.tabs.get(chrome._mangoTabId).then(function (mainTab) {
            sendMessage(mainTab);
        }, function (err) {
            //mainTab已关闭
            createMainTab();
        });
    } else {
        createMainTab();
    }
    
    function sendMessage (mainTab) {
        //t延时1秒，否则会报错
        setTimeout(function () {
            chrome.tabs.sendMessage(mainTab.id, {
                ID: 'mainTab',
                site: site,
                type: type,
                title: tab.title,
                url: pageUrl
            });
        }, 1000);
        if (type === 'single' && silent !== true && tab.id) {
            chrome.tabs.sendMessage(tab.id, {ID:'content', action:'notify', url:pageUrl});
        }
    }
    function createMainTab () {
        var main = chrome.extension.getURL("main.htm");
        /*
         * 当分类采集时，默认选中tab
         * 单品采集时，默认不选中tab
         */
        chrome.tabs.create({'url': main, active:type === 'category'}).then(function (mainTab) {
            if (type === 'single') {
                chrome._mangoTabId = mainTab.id;
            }
            sendMessage(mainTab);
        });
    }
}

