var FETCH_HOST = 'fetch.mangoerp.com';
var runtime = browser.runtime;
var chrome = browser;
var CONFIG, OPTIONS = {};
runtime.onMessage.addListener(function (message, sender, callback) {
    if (message.ID !== 'content') {
        return;
    }
    switch (message.action) {
        case 'notify':
            notify(message.msg);
            break;
        case 'handleLinks':
            debounceHandleLinks();
            break;
    }
});

if (location.href.indexOf('www.mangoerp.com/erp') !== -1) {
    //和webpage通信，接收外部消息
    exportFunction(function (opt) {
        opt.ID = 'background';
        opt.action = 'mangoerp.' + opt.action;
        //将消息发送到background.js
        runtime.sendMessage(opt);
    }, window, {defineAs: 'mangoFirefoxExtension'});
}

if (document.contentType === 'text/html') {
    chrome.storage.sync.get({
        showPageFetchBar: true,
        showProductFetchBtn: true
    }).then(function (data) {
        OPTIONS = data;
        preload();
    });
}

var debounceHandleLinks = debounce(function () {
    if (CONFIG && CONFIG.detail) {
        handleLinks(CONFIG.detail);
    }
}, 800);

//获取当前域名配置信息
function preload () {
    $.ajax({
        type: 'get', 
        url: '//' + FETCH_HOST + '/parse/identify?url=' + encodeURIComponent(location.href), 
        success: function (res) {
            if (!res.success) {
                return;
            }
            CONFIG = res.data;
            try {
                var adapterId = (new Function('$', 'url', CONFIG.detect))($, location.href);
                var pageType = CONFIG.map[adapterId];
                if (pageType) {
                    if (OPTIONS.showPageFetchBar) {
                        showTip(pageType);
                    }
                    runtime.sendMessage({ID:'background', action:'preload', type:pageType});
                }
                handleLinks(CONFIG.detail);
                
                $(window).scroll(debounceHandleLinks);
            } catch (e) {
                console.log('mango.preload', e);
            }
        }
    });
}

//增加芒果采集按钮
function handleLinks (fnBody) {
    if (!OPTIONS.showProductFetchBtn) {
        return;
    }
    console.time('mangofetch.links');
    $.each(document.links, function (index, a) {
        if (a.dataset.mango) {
            return;
        }
        a.dataset.mango = true;
        var href = a.href;
        //如果是相对地址，浏览器会自动变成完整地址。
        //getAttribute获取的是原始地址
        if (!href) {
            return;
        }
        if (a.getAttribute('href') === '#') {
            return;
        }
        if (href.indexOf('javascript:') === 0) {
            return;
        }
        if (href.indexOf('//') === 0) {
            href = location.protocol + href;
        }
        try {
            var test = (new Function('url', fnBody))(href);
            if (test && validArea(a)) {
                var $a = $(a);
                insertFetchBtn($a, href);
            }
        } catch (e) {
            console.log(e);
        }
    });
    console.timeEnd('mangofetch.links');

    if (CONFIG.selectors) {
        CONFIG.selectors.forEach(function (item) {
            var getUrl = new Function('$el', item.fn);
            $(item.css).each(function (index, el) {
                if (el.dataset.mango) {
                    return;
                }
                el.dataset.mango = true;
                insertFetchBtn($(el), getUrl);
            });
        });
    }
}

function insertFetchBtn($a, url) {
    var $mango;
    $a.addClass('mango-link');
    $mango = $('<span class="mango-link-next">采集到芒果</span>').insertAfter($a);
    
    $a.one('mouseenter', function () {
        /*
         * 使用$a.position 获取位置
         * 使用$img.width() 获取width，因为a如果未设置display:block，width可能为0
         *
         */
        var $firstImg = $a.children('img').first();
        var pos, $img, width, position;
        
        if (CONFIG.marginTop) {
            $mango.css('margin-top', CONFIG.marginTop);
        }
        if (CONFIG.marginLeft) {
            $mango.css('margin-left', CONFIG.marginLeft);
        }
        if ($firstImg.length && $a.css('position') === 'static' && $firstImg.css('display') !== 'none') {
            /*
             * a{position:static} 
             * img{display:block}
             * 时使用图片position
             *
             */
            pos = $firstImg.position();
            width = $firstImg.width();
        } else {
            if (checkSiblingAllA($a)) {
                pos = $a.children().first().position();
            } else {
                position = $a.parent().css('position');
                if (['relative', 'absolute'].indexOf(position) !== -1) {
                    //使用css定位
                    return;
                }
                pos = $a.position();
            }
            $img = $a.find('img');
            //判断图片是否已经显示
            if ($img.css('display') !== 'none') {
                width = $img.width();
            }
        }

        if (!width || width < 80) {
            width = $a.width();
        }
        
        if (width < 80) {
            $mango.html('采集');
        }
        
        if (width < 60) {
            $mango.remove();
            return;
        }

        pos.left += (width || 0) / 2;
        pos.top += 20;
        $mango.css(pos);
    });
    $mango.click(function () {
        var $span = $(this);
        if (!$span.hasClass('success')) {
            var alt = $a.find('img').attr('alt');

            if (typeof(url) === 'function') {
                try {
                    url = url($a);
                } catch (err) {
                    err.id = 'extension.product_url_miss';
                    $.ajax('//' + FETCH_HOST + '/parse/error', {fetchUrl:location.href, err:JSON.stringify(err)});
                    return notify('采集出错，我们会尽快修复!', 'error');
                }
            }
            runtime.sendMessage({ID:'background', action:'fetch', type:'single', url:url, title:alt || url, silent:true});
            notify();
            $span.text('已采集').addClass('success');
        }
    });
}

/*
 * 当div>a*5>div>img
 * 这种结构时$a.position位置显示错误
 *
 */
function checkSiblingAllA ($a) {
    var doms = $a.siblings();
    if (doms.length <= 2) {
        return false;
    }
    var $el, el;
    for (var i = 0, len = doms.length; i < len; i++) {
        el = doms[i];
        var $el = $(el);
        if ($el.hasClass('mango-link-next')) {
            continue;
        }
        if (el.tagName.toLowerCase() !== 'a') {
            return false;
        }
    }
    return true;
}
function showTip (type) {
    var html = [
      '<div class="mango-fetch">',
        '芒果店长：当前页面支持',
        (type === 'category' ? '分类' : '单品') + '采集，',
        '<a action="fetch" class="m-btn" href="javascript:void(0);">开始采集</a>',
        '<a class="options" action="options" href="javascript:void(0);">选项</a>',
        '<span class="close" title="关闭" action="close">x</span>',
        // '<a href="https://www.mangoerp.com/" target="_blank" class="logo"><img src="//asset.mangoerp.com/img/icon/mangoerp.png" alt="芒果店长" /></a>',
      '</div>'
    ];
    var host = location.host;

    if (type === 'single') {
      if (host === 'detail.1688.com') {
        html.splice(4, 0, '<a action="discount" class="m-btn m-btn-warning" href="javascript:void(0);">找低价</a>')
      }
      if (!(/(taobao|tmall|1688)\.com$/.test(host))) {
        html.splice(4, 0, '<a action="supply" class="m-btn m-btn-warning" href="javascript:void(0);">找货源</a>')
      }
    }
    $(html.join('')).appendTo('body').click(function (ev) {
        var $el = $(this);
        var action = $(ev.target).attr('action');
        switch (action) {
            case 'fetch':
                runtime.sendMessage({ID:'background', action:'fetch', type:type, silent:true});
                notify();
                $el.remove();
                break;
            case 'discount':
                window.open('https://www.mangoerp.com/erp/ali1688/shipping/opendijia?url=' + encodeURIComponent(location.href))
                $el.remove();
                break;
            case 'supply':
                window.open('https://www.mangoerp.com/erp/ali1688/shipping/opentongkuan?url=' + encodeURIComponent(location.href))
                break;
            case 'close':
                $el.remove();
                break;
            case 'options':
                runtime.sendMessage({ID:'background', action:'options'});
                break;
        }
    });
};

//当前链接是否为有效链接
function validArea (a) {
    var $a = $(a);
    return $a.find('img').length || $a.css('background-image') !== 'none' || $a.html().indexOf('background') !== -1;
}

//显示通知
function notify (msg, type) {
    msg = msg || '开始采集，您可以在采集中心查看结果';

    var html = '<div class="mango-notify ' + (type ? type : '') + '">' + msg + '</div>';
    $(html).appendTo('body').fadeIn(function () {
        var $el = $(this);
        setTimeout(function () {
            $el.fadeOut(function () {
                $el.remove();
            });
        }, 2000);
    });
}
function debounce (func, wait, immediate) {
    var timeout;
    return function() {
        var context = this, args = arguments;
        var later = function() {
            timeout = null;
            if (!immediate) {
                func.apply(context, args);
            }
        };
        var callNow = immediate && !timeout;
        clearTimeout(timeout);
        timeout = setTimeout(later, wait);
        if (callNow) {
            func.apply(context, args);
        }
    };
};
